// TCP_IP.cpp : Defines the exported functions for the DLL application.
//
/////////////////////////////////////////////////////////////////////////////
// Changelog
//
//    date  | by | description
// ---------+----+-------------------------------------------------------------
//          |    |
//   260116 | SB | Modified the file to handle 255 ports at once and to remove all multithreading
//          |    |


#include "stdafx.h"
#include <windows.h>
#include <tchar.h>

#include <WinIoCtl.h>
#include <InitGuid.h>
#include <setupapi.h>
#include <devguid.h>
#include <regstr.h>
#include <Setupapi.h>

#include "FASlave.h"

#define WIN32_LEAN_AND_MEAN

#define MAX_COM_PORTS   255
 
extern "C"
{
  class ComPort 
  {
  public:
    ComPort()
    { 
      memset(&dcb, 0, sizeof(DCB));
      hPort = INVALID_HANDLE_VALUE;
    }
    DCB dcb;
    HANDLE hPort;
  };

  // Array of ports (to enable using port as index)
  ComPort ComPorts[MAX_COM_PORTS];


/*=----------------------------------------------------------------------=*\
  Name :GetPortDeviceSet
  Desc :Returns a handle to the "Ports" Device Set
  In   :  None
  Out  :  None
  Retn :  HDEVINFO handle
\*=----------------------------------------------------------------------=*/
HDEVINFO GetPortDeviceSet()
{
  HDEVINFO hDevices = INVALID_HANDLE_VALUE;
  DWORD dwGuids = 0;
  BOOL bRes = SetupDiClassGuidsFromName(_T("Ports"), 0, 0, &dwGuids);
  if (dwGuids)
  {
    LPGUID pguids = (LPGUID)HeapAlloc(GetProcessHeap(), 0, sizeof(GUID) * dwGuids);
    if (pguids)
    {
      bRes = SetupDiClassGuidsFromName(_T("Ports"), pguids, dwGuids, &dwGuids);
      if (bRes)
      {
        hDevices = SetupDiGetClassDevs(pguids, NULL, NULL, DIGCF_PRESENT);
      }
      HeapFree(GetProcessHeap(), 0, pguids);
    }
  }
  return hDevices;
}



bool EnumerateComsPorts(bool Refresh)
{
  bool retenum = false;

  if( m_ComsPorts.IsEmpty() || Refresh )
  {
    m_ComsPorts.RemoveAll();
    //set the port array size based on number of ports present
    m_ComsPorts.SetSize(0);
    //obtain device information set for COM ports
    HDEVINFO hDevInfo = GetPortDeviceSet();

    SP_DEVINFO_DATA DeviceInfoData;
    ZeroMemory(&DeviceInfoData, sizeof(SP_DEVINFO_DATA));
    DeviceInfoData.cbSize = sizeof(SP_DEVINFO_DATA);
    DWORD DeviceIndex = 0;
    DWORD CommsIdx = 0;
    bool bPortIsSerial;

    while( SetupDiEnumDeviceInfo(hDevInfo, DeviceIndex, &DeviceInfoData) )
    {
      PortDetails portinfo;
      retenum = true; //seen something
      bPortIsSerial = false;

		  // Enumerate the current device
		  DeviceInfoData.cbSize = sizeof( SP_DEVINFO_DATA);
      // Get the registry key which stores the port settings
      HKEY hDeviceKey = SetupDiOpenDevRegKey( hDevInfo, &DeviceInfoData, DICS_FLAG_GLOBAL, 0, DIREG_DEV, KEY_QUERY_VALUE);
      if( hDeviceKey)
      {
        // Read in the name of the port
        TCHAR pszPortName[20];
        DWORD dwSize = sizeof(pszPortName);
        DWORD dwType = 0;
        if( (RegQueryValueEx(hDeviceKey, _T("PortName"), NULL, &dwType, (LPBYTE) pszPortName, &dwSize) == ERROR_SUCCESS) && (dwType == REG_SZ) )
        {
          // Check if it really is a com port
          if( _tcsnicmp( pszPortName, _T("COM"), 3) == 0)
          {
            int nPortNr = _ttoi( pszPortName + 3 );
            if( nPortNr != 0 )
            {
              portinfo.PortName = pszPortName;
              bPortIsSerial = true;
              //Got the port name, now get a property: Friendly name
              DWORD bfrsize = 0;
              LPTSTR buffer = NULL;
              while (!SetupDiGetDeviceRegistryProperty(hDevInfo, &DeviceInfoData, SPDRP_DEVICEDESC, NULL, (PBYTE)buffer, bfrsize, &bfrsize))
              {
                if (GetLastError() == ERROR_INSUFFICIENT_BUFFER)
                {
                  // Change the buffer size.
                  if (buffer) LocalFree(buffer);
                  // Double the size to avoid problems on
                  // W2k MBCS systems
                  buffer = (LPTSTR)LocalAlloc(LPTR, bfrsize * 2);
                  ZeroMemory(buffer, bfrsize*2);
                }
                else
                {
                  // other errors ... don't care
                  break;
                }
              }

              if( buffer!=NULL && _tcslen(buffer) > 0 )
              {
                //if property found store it in the port details
                portinfo.DevDesc.Format(_T("%s"), buffer);
                LocalFree(buffer);
              }

              //repeat for property: HardwareID
              bfrsize = 0;
              buffer = NULL;
              while (!SetupDiGetDeviceRegistryProperty(hDevInfo, &DeviceInfoData, SPDRP_HARDWAREID, NULL, (PBYTE)buffer, bfrsize, &bfrsize))
              {
                if (GetLastError() == ERROR_INSUFFICIENT_BUFFER)
                {
                  // Change the buffer size.
                  if (buffer) LocalFree(buffer);
                  // Double the size to avoid problems on
                  // W2k MBCS systems
                  buffer = (LPTSTR)LocalAlloc(LPTR, bfrsize * 2);
                  ZeroMemory(buffer, bfrsize*2);
                }
                else
                {
                  // other errors ... don't care
                  break;
                }
              }

              if( buffer!=NULL && _tcslen(buffer) > 0 )
              {
                //if property found store it in the port details
                TCHAR *pBfr = _tcsstr(buffer, _T("VID_"));
                if( pBfr )
                {
                  portinfo.HardwareID.VendorID.Format(_T("%.4s"), pBfr+4);
                  portinfo.HardwareID.VendorID.MakeUpper();

                  pBfr = _tcsstr(buffer, _T("PID_"));
                  if( pBfr )
                  {
                    portinfo.HardwareID.ProductID.Format(_T("%.4s"), pBfr+4);
                    portinfo.HardwareID.ProductID.MakeUpper();
                    portinfo.HardwareID.IdSet = true;
                  }
                }
                LocalFree(buffer);
              }
            }
          }
        }
		  }
		  // Close the key
		  RegCloseKey(hDeviceKey);
      if( bPortIsSerial )
      {
        //store port details in array & grow if necessary (allow for pnp devices)
        m_ComsPorts.SetAtGrow(CommsIdx, portinfo);
        CommsIdx++;
      }
      DeviceIndex++;
    }

    if (hDevInfo)
    {
      SetupDiDestroyDeviceInfoList(hDevInfo);
    }
	}
  else
  {
    //array already populated
    retenum = true;
  }

  return retenum;
}







  /*=----------------------------------------------------------------------=*\
     Use :FA_LEDWrite
         :Sets the value of the LEDs as an 8-bit output
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_LEDWrite( unsigned char Port, unsigned char value )
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 0);   //Macro
      SendCharCOM(Port, value);   //Data0
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_LEDOn
         :Sets a single LED 0-7
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_LEDOn( unsigned char Port, unsigned char index )
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 1);   //Macro
      SendCharCOM(Port, index);   //Data0
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_LEDOff
         :Clears a single LED 0-7
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_LEDOff( unsigned char Port, unsigned char index )
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 2);   //Macro
      SendCharCOM(Port, index);   //Data0
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_ReadSwitch
         :Reads the user switches
         :index: 0=Left, 1=Right
         :0=NoPress, 1=Press
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned char FA_ReadSwitch( unsigned char Port, unsigned char index )
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 3);   //Macro
      SendCharCOM(Port, index);   //Data0
      return ReadCharCOM(Port, 100, 256);   //Return result
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_WaitSwitch
         :Waits for a switch press
         :a: 1=Left, 2=Right, 3=Both
  \*=----------------------------------------------------------------------=*/
  /*
  DECLDIR unsigned char FA_WaitSwitch( unsigned char Port, unsigned char a )
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 4);   //Macro
      SendCharCOM(Port, a);   //Data0
      return ReadCharCOM(Port, 100, 256);   //Return result
  }*/

  /*=----------------------------------------------------------------------=*\
     Use :FA_SetMotors
         :Sets the motor speeds without feedback
         :Range -100 - 100
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_SetMotors( unsigned char Port, signed int left, signed int right )
  {
      //Simple range check
      /*
      if (left > 100)
      {
        left = 100;
      }
      else if (left < -100)
      {
        left = -100;
      }
      if (right > 100)
      {
        right = 100;
      }
      else if (right < -100)
      {
        right = -100;
      }  
      */
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 5);   //Macro
      SendCharCOM(Port, left & 0xFF);   //Data0 Left LSB
      SendCharCOM(Port, left >> 8);     //Data1 Left MSB
      SendCharCOM(Port, right & 0xFF);   //Data2 Right LSB
      SendCharCOM(Port, right >> 8);     //Data3 Right MSB
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_ReadIR
         :Reads one of the IR distance sensors 0-7
         :a: 0=Left, 1=FrontLeft, 2=Front, 3=FrontRight, 4=Right, 5=RearRight, 6=Rear, 7=RearLeft
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned int FA_ReadIR( unsigned char Port, unsigned char index )
  {
      unsigned char MSB, LSB;
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 6);   //Macro
      SendCharCOM(Port, index);   //Data0
      LSB = ReadCharCOM(Port, 1000, 256);   //Return result
      MSB = ReadCharCOM(Port, 1000, 256);   //Return result
      return ((MSB << 8) + LSB);    
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_ReadLine
         :Reads one of the IR line sensors 0-1
         :a: 0=Left, 1=Right
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned int FA_ReadLine( unsigned char Port, unsigned char index )
  {
      unsigned char MSB, LSB;
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 7);   //Macro
      SendCharCOM(Port, index);   //Data0
      LSB = ReadCharCOM(Port, 1000, 256);   //Return result
      MSB = ReadCharCOM(Port, 1000, 256);   //Return result
      return ((MSB << 8) + LSB);    
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_ReadLight
         :Reads the light sensor
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned int FA_ReadLight( unsigned char Port )
  {
      unsigned char MSB, LSB;
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 8);   //Macro
      LSB = ReadCharCOM(Port, 1000, 256);   //Return result
      MSB = ReadCharCOM(Port, 1000, 256);   //Return result
      return ((MSB << 8) + LSB);    
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_Forwards
         :Drives the robot forwards - waits for the distance to be traversed before returning
         :a: distance to travel in mm
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_Forwards( unsigned char Port, unsigned int distance )
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 9);   //Macro
      SendCharCOM(Port, distance & 0xFF);    //Distance LSB
      SendCharCOM(Port, distance >> 8);      //Distance MSB
      ReadCharCOM(Port, 20000, 256);  //Wait up to 20 seconds for return
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_Backwards
         :Drives the robot backwards - waits for the distance to be traversed before returning
         :a: distance to travel in mm
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_Backwards(unsigned char Port, unsigned int distance)
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 10);   //Macro
      SendCharCOM(Port, distance & 0xFF);    //Distance LSB
      SendCharCOM(Port, distance >> 8);      //Distance MSB
      ReadCharCOM(Port, 20000, 256);  //Wait up to 20 seconds for return
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_Left
         :Rotates the robot left - waits for the angle to be traversed before returning
         :a: angle to travel in degrees
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_Left(unsigned char Port, unsigned int angle)
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 11);   //Macro
      SendCharCOM(Port, angle & 0xFF);    //Distance LSB
      SendCharCOM(Port, angle >> 8);      //Distance MSB
      ReadCharCOM(Port, 20000, 256);  //Wait up to 20 seconds for return
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_Right
         :Rotates the robot right - waits for the angle to be traversed before returning
         :a: angle to travel in degrees
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_Right(unsigned char Port, unsigned int angle)
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 12);   //Macro
      SendCharCOM(Port, angle & 0xFF);    //Distance LSB
      SendCharCOM(Port, angle >> 8);      //Distance MSB
      ReadCharCOM(Port, 20000, 256);  //Wait up to 20 seconds for return
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_SetLogoSpeed
         :Sets the speed of logo movements
         :a: Logo Speed 1-100
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_SetLogoSpeed(unsigned char Port, unsigned char speed)
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 13);  //Macro
      SendCharCOM(Port, speed);   //Speed
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_EncoderReset
         :Resets the motor encoder counters
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_EncoderReset(unsigned char Port)
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 14);  //Macro
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_EncoderRead
         :Reads one of the motor encoder counters
         :a: 0=Left, 1=Right
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned int FA_EncoderRead(unsigned char Port, unsigned char index)
  {
      unsigned char MSB, LSB;
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 15);  //Macro
      SendCharCOM(Port, index);   //Encoder
      LSB = ReadCharCOM(Port, 100, 256);   //Return result
      MSB = ReadCharCOM(Port, 100, 256);   //Return result
      return ((MSB << 8) + LSB);    
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_LCDClear
         :Clears the LCD
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_LCDClear(unsigned char Port)
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 16);  //Macro
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_LCDPrintString
         :Prints a string to the LCD
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_LCDPrintString(unsigned char Port, unsigned char x, unsigned char y, unsigned char * text)
  {
      unsigned char idx = 0;
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 17);  //Macro
      SendCharCOM(Port, x);  //X
      SendCharCOM(Port, y);  //Y
      while (*text > 0)
      {
        SendCharCOM(Port, *text++);
        idx++;
      }    
      SendCharCOM(Port, 0);   //String termination
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_LCDPrintNumber
         :Prints a 16-bit signed number to the LCD
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_LCDPrintNumber(unsigned char Port, unsigned char x, unsigned char y, signed int number)
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 18);  //Macro
      SendCharCOM(Port, x);   //X
      SendCharCOM(Port, y);          //Line
      SendCharCOM(Port, number & 0xFF);    //Number LSB
      SendCharCOM(Port, number >> 8);      //Number MSB
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_LCDDrawPixel
         :Allows a single pixel on the LCD to be set or cleared
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_LCDDrawPixel(unsigned char Port, unsigned char x, unsigned char y, unsigned char state)
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 19);  //Macro
      SendCharCOM(Port, x);   //X
      SendCharCOM(Port, y);   //Y
      SendCharCOM(Port, state);    //Pixel State
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_LCDDrawLine
         :Allows a single pixel width line to be drawn on the LCD
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_LCDDrawLine(unsigned char Port, unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2)
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 20);  //Macro
      SendCharCOM(Port, x1);   //X
      SendCharCOM(Port, y1);   //Y
      SendCharCOM(Port, x2);   //X
      SendCharCOM(Port, y2);   //Y
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_LCDDrawRect
         :Allows a rectangle to be drawn on the LCD
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_LCDDrawRect(unsigned char Port, unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2)
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 21);  //Macro
      SendCharCOM(Port, x1);   //X
      SendCharCOM(Port, y1);   //Y
      SendCharCOM(Port, x2);   //X
      SendCharCOM(Port, y2);   //Y
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_LCDBacklight
         :Allows the LCD backlight brighntess to be adjusted
         :a: Range 0-100, 0=Off, 1=Min Brightness, 100=Max Brightness
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_LCDBacklight(unsigned char Port, unsigned char value)
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 22);  //Macro
      SendCharCOM(Port, value);   //Brightness
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_LCDVerbose
         :Allows the LCD to report the API commands or used by the user - default verbose on
         :a: 0=Off 1=On
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_LCDVerbose(unsigned char Port, unsigned char value)
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 23);  //Macro
      SendCharCOM(Port, value);   //Brightness
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_LCDOptions
         :Allows the LCD to set the foreground and background pixel colours and transparancy
         :a: 0=Off 1=On
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_LCDOptions(unsigned char Port, unsigned char ForeColour, unsigned char BackColour, unsigned char Transparent)
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 27);  //Macro
      SendCharCOM(Port, ForeColour);   //FG
      SendCharCOM(Port, BackColour);   //BG
      SendCharCOM(Port, Transparent);   //Trans
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_ReadMic
         :Allows the LCD to report the API commands or used by the user - default verbose on
         :a: 0=Off 1=On
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned int FA_ReadMic(unsigned char Port)
  {
      unsigned char MSB, LSB;
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 24);  //Macro
      LSB = ReadCharCOM(Port, 100, 256);   //Return result
      MSB = ReadCharCOM(Port, 100, 256);   //Return result
      return ((MSB << 8) + LSB); 
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_PlayNote
         :Plays a note using the speaker on the FA
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_PlayNote(unsigned char Port, unsigned int note, unsigned int time)
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 25);  //Macro
      SendCharCOM(Port, note & 0xFF);   //Brightness
      SendCharCOM(Port, note >> 8);   //Brightness
      SendCharCOM(Port, time & 0xFF);    //Number LSB
      SendCharCOM(Port, time >> 8);      //Number MSB
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_PlayNote
         :Plays a note using the speaker on the FA
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned char FA_GetAPIVersion(unsigned char Port)
  {
      SendCharCOM(Port, 0);   //Component
      SendCharCOM(Port, 26);  //Macro
      return ReadCharCOM(Port, 100, 256);   //Return result
  }














  /*=----------------------------------------------------------------------=*\
     Use :FA_ServoEnable
         :Allows one of the servo outputs 0-3 to be enabled
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_ServoEnable(unsigned char Port, unsigned char index)
  {
      SendCharCOM(Port, 1);   //Component
      SendCharCOM(Port, 0);   //Macro
      SendCharCOM(Port, index);   //Channel 0-3
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_ServoDisable
         :Allows one of the servo outputs 0-3 to be disabled
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_ServoDisable(unsigned char Port, unsigned char index)
  {
      SendCharCOM(Port, 1);   //Component
      SendCharCOM(Port, 1);   //Macro
      SendCharCOM(Port, index);   //Channel 0-3
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_ServoSetPosition
         :Allows one of the servo positions to be assigned
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_ServoSetPosition(unsigned char Port, unsigned char index, unsigned char position)
  {
      SendCharCOM(Port, 1);   //Component
      SendCharCOM(Port, 2);   //Macro
      SendCharCOM(Port, index);   //Channel 0-3
      SendCharCOM(Port, position);   //Position 0-255
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_ServoAutoMoveToPosition
         :Allows one of the servo positions to move gradually to a new position
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_ServoAutoMoveToPosition(unsigned char Port, unsigned char index, unsigned char position)
  {
      SendCharCOM(Port, 1);   //Component
      SendCharCOM(Port, 3);   //Macro
      SendCharCOM(Port, index);   //Channel 0-3
      SendCharCOM(Port, position);   //Position 0-255
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_ServoSetAutoMoveSpeed
         :Changes the speed of the auto movements, default 1
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_ServoSetAutoMoveSpeed(unsigned char Port, unsigned char speed)
  {
      SendCharCOM(Port, 1);   //Component
      SendCharCOM(Port, 4);   //Macro
      SendCharCOM(Port, speed);   //Speed 1-50
  }


















  /*=----------------------------------------------------------------------=*\
     Use :FA_CardInit
         :Attempts to startup the SD card, 255=No Card, 254=Init Fail, 0=Init OK
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned char FA_CardInit( unsigned char Port )
  {
      SendCharCOM(Port, 2);   //Component
      SendCharCOM(Port, 0);   //Macro
      return (ReadCharCOM(Port, 100, 256));   //Return result
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_CardCreate
         :Attempts to create a file on the SD card, 255=Error, 1=File Already Exists, 0=Create OK
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned char FA_CardCreate( unsigned char Port, unsigned char * filename )
  {
      unsigned char idx = 0;
      SendCharCOM(Port, 2);   //Component
      SendCharCOM(Port, 1);   //Macro
      while (filename[idx] > 0)
      {
        SendCharCOM(Port, filename[idx]);
        idx++;
      }    
      SendCharCOM(Port, 0);   //String termination
      return (ReadCharCOM(Port, 100, 256));   //Return result
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_CardOpen
         :Attempts to open an existing file on the SD card, 255=Error, 1=File not found, 0=Open OK
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned char FA_CardOpen( unsigned char Port, unsigned char * filename )
  {
      unsigned char idx = 0;
      SendCharCOM(Port, 2);   //Component
      SendCharCOM(Port, 2);   //Macro
      while (filename[idx] > 0)
      {
        SendCharCOM(Port, filename[idx]);
        idx++;
      }    
      SendCharCOM(Port, 0);   //String termination
      return (ReadCharCOM(Port, 100, 256));   //Return result
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_CardDelete
         :Attempts to delete an existing file on the SD card, 255=Error, 1=File not found, 0=Delete OK
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned char FA_CardDelete( unsigned char Port, unsigned char * filename )
  {
      unsigned char idx = 0;
      SendCharCOM(Port, 2);   //Component
      SendCharCOM(Port, 3);   //Macro
      while (filename[idx] > 0)
      {
        SendCharCOM(Port, filename[idx]);
        idx++;
      }    
      SendCharCOM(Port, 0);   //String termination
      return (ReadCharCOM(Port, 100, 256));   //Return result
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_CardWriteByte
         :Appends a byte to the end of the currently open file 
  \*=----------------------------------------------------------------------=*/
  DECLDIR void FA_CardWriteByte( unsigned char Port, unsigned char data )
  {
      SendCharCOM(Port, 2);   //Component
      SendCharCOM(Port, 4);   //Macro
      SendCharCOM(Port, data);   //data byte
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_CardReadByte
         :Reads a byte from the currently open file, starts at the beginning of the file and auto increments to the end
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned char FA_CardReadByte( unsigned char Port )
  {
      SendCharCOM(Port, 2);   //Component
      SendCharCOM(Port, 5);   //Macro
      return (ReadCharCOM(Port, 100, 256));   //Return result
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_CardRecordMic
         :Records an audio stream on the card from the microphone
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned char FA_CardRecordMic( unsigned char Port, unsigned char bitdepth, unsigned char samplerate, unsigned int time, unsigned char * filename)
  {
      unsigned char idx = 0;
      SendCharCOM(Port, 2);   //Component
      SendCharCOM(Port, 6);   //Macro
      SendCharCOM(Port, bitdepth);   
      SendCharCOM(Port, samplerate);    
      SendCharCOM(Port, time & 0xFF);    //Number LSB
      SendCharCOM(Port, time >> 8);      //Number MSB
      while (filename[idx] > 0)
      {
        SendCharCOM(Port, filename[idx]);
        idx++;
      }    
      SendCharCOM(Port, 0);   //String termination
      return (ReadCharCOM(Port, 60000, 256));   //Return result
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_CardPlayback
         :Plays back a .Wav file audio stream from the card
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned char FA_CardPlayback(unsigned char Port, unsigned char * filename)
  {
      unsigned char idx = 0;
      SendCharCOM(Port, 2);   //Component
      SendCharCOM(Port, 7);   //Macro
      while (filename[idx] > 0)
      {
        SendCharCOM(Port, filename[idx]);
        idx++;
      }    
      SendCharCOM(Port, 0);   //String termination
      return (ReadCharCOM(Port, 60000, 256));   //Return result
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_CardBitmap
         :Displays a monochrome bitmap read from the card on the LCD
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned char FA_CardBitmap(unsigned char Port, unsigned char x, unsigned char y, unsigned char * filename)
  {
      unsigned char idx = 0;
      SendCharCOM(Port, 2);   //Component
      SendCharCOM(Port, 8);   //Macro
      SendCharCOM(Port, x);   //Macro
      SendCharCOM(Port, y);   //Macro
      while (filename[idx] > 0)
      {
        SendCharCOM(Port, filename[idx]);
        idx++;
      }    
      SendCharCOM(Port, 0);   //String termination
      return (ReadCharCOM(Port, 60000, 256));   //Return result
  }

























  /*=----------------------------------------------------------------------=*\
     Use :FA_ReadAxis
         :Reads a single X/Y/Z axis from the last accelerometer or magnetometer sample
         :axis: 0=X, 1=Y, 2=Z
  \*=----------------------------------------------------------------------=*/
  DECLDIR signed int FA_ReadAxis(unsigned char Port, unsigned char index)
  {
      unsigned char MSB, LSB;
      SendCharCOM(Port, 3);   //Component
      SendCharCOM(Port, 0);   //Macro
      SendCharCOM(Port, index);   //axis
      LSB = ReadCharCOM(Port, 100, 256);   //Return result
      MSB = ReadCharCOM(Port, 100, 256);   //Return result
      return ((MSB << 8) + LSB); 
  }

  /*=----------------------------------------------------------------------=*\
     Use :FA_ReadBearing
         :Reads the compass bearing based on a number of samples
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned int FA_ReadBearing( unsigned char Port )
  {
      unsigned char MSB, LSB;
      SendCharCOM(Port, 3);   //Component
      SendCharCOM(Port, 1);   //Macro
      LSB = ReadCharCOM(Port, 100, 256);   //Return result
      MSB = ReadCharCOM(Port, 100, 256);   //Return result
      return ((MSB << 8) + LSB); 
  }

















  /*=----------------------------------------------------------------------=*\
     Use :Returns the port number of the first port to respond as a valid FA
  \*=----------------------------------------------------------------------=*/
  DECLDIR unsigned char FA_ComFindFirst( void )
  {
    unsigned char retval = 1;
    unsigned char found = 0;
    
    while ((found == 0) && (retval < MAX_COM_PORTS))
    {
      found = FA_ComQuery(retval);
      
      if (found == 0)
        retval++;
    }
      
    if (found == 1)
      return (retval);
    else
      return (0);
  }








/*=----------------------------------------------------------------------=*\
   Use :Check whether a COM port exists in device manager
       :
\*=----------------------------------------------------------------------=*/
	DECLDIR unsigned char FA_ComQuery( unsigned char Port )
	{
	    //Populate list when Port = 1, then no need to keep re-populating.
       if (Port == 1)
       {
          EnumerateComsPorts(true);
       }
       else
       {
          EnumerateComsPorts(false);
       }

	   TCHAR  combuffer[64];
       _stprintf_s(combuffer, 64, _T("COM%d"), Port);

       for (int x = 0; x < m_ComsPorts.GetSize(); x++)
       {
          if ( m_ComsPorts.GetAt(x).PortName.Compare(combuffer) == 0)
              return true;
       }

       return false;


      //----- No longer called -----
      /*
        if (Port < 10)
        {
          _stprintf_s(combuffer, 64, _T("COM%d"), Port);
        }
        else
        {
          _stprintf_s(combuffer, 64, _T("\\\\.\\COM%d"), Port);
        }

        ComPorts[Port].hPort = CreateFile(combuffer, GENERIC_WRITE|GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_WRITE, 0, OPEN_EXISTING, 0, NULL); 
       
        if (ComPorts[Port].hPort==INVALID_HANDLE_VALUE)
        {
          return false; 
        }
        else
        {
          CloseHandle(ComPorts[Port].hPort);
          ComPorts[Port].hPort = INVALID_HANDLE_VALUE;
        }
        
        return true;
      */
    
	}


/*=----------------------------------------------------------------------=*\
   Use :Opens the specified COM port as the stream to use
       :
\*=----------------------------------------------------------------------=*/
	DECLDIR unsigned char FA_ComOpen( unsigned char Port )
	{
    unsigned char retval = 0;
	  TCHAR  combuffer[64];

    if (Port < 10)
    {
      _stprintf_s(combuffer, 64, _T("COM%d"), Port);
    }
    else
    {
      _stprintf_s(combuffer, 64, _T("\\\\.\\COM%d"), Port);
    }
 
    ComPorts[Port].hPort = CreateFile(combuffer, GENERIC_WRITE | GENERIC_READ, 0, 0, OPEN_EXISTING, 0, 0); 
    
    ComPorts[Port].dcb.DCBlength=sizeof(DCB);
    if (!GetCommState(ComPorts[Port].hPort, &ComPorts[Port].dcb)) 
    {
      //error getting state
      retval = 255;
    }
    
    ComPorts[Port].dcb.BaudRate=CBR_9600;
    ComPorts[Port].dcb.ByteSize=8;
    ComPorts[Port].dcb.StopBits=ONESTOPBIT;
    ComPorts[Port].dcb.Parity=NOPARITY;
    
    if(!SetCommState(ComPorts[Port].hPort, &ComPorts[Port].dcb))
    {
      //error setting serial port state
      retval = 255;
    }
    
    COMMTIMEOUTS timeouts={0};
    timeouts.ReadIntervalTimeout=50;
    timeouts.ReadTotalTimeoutConstant=50;
    timeouts.ReadTotalTimeoutMultiplier=10; 
    timeouts.WriteTotalTimeoutConstant=50;
    timeouts.WriteTotalTimeoutMultiplier=10;
    if(!SetCommTimeouts(ComPorts[Port].hPort, &timeouts))
    {
      //error occureed. Inform user
      retval = 255;
    }

    return (retval);
  }


/*=----------------------------------------------------------------------=*\
   Use :Closes a previously open COM port
       :
\*=----------------------------------------------------------------------=*/
	DECLDIR unsigned char FA_ComClose( unsigned char Port )
	{
	  if (ComPorts[Port].hPort != INVALID_HANDLE_VALUE)
	  {
	    CloseHandle(ComPorts[Port].hPort);
	    ComPorts[Port].hPort = INVALID_HANDLE_VALUE; // Invalid handle now
	  }

    return 0;
	}


/*=----------------------------------------------------------------------=*\
   Use :Sets the baud rate for the currently open COM
       :
\*=----------------------------------------------------------------------=*/
	DECLDIR void BaudRateSet( unsigned char Port, unsigned long Baud )
	{
    ComPorts[Port].dcb.DCBlength = sizeof(DCB);
    if (!GetCommState(ComPorts[Port].hPort, &ComPorts[Port].dcb)) 
    {
      //error getting state
    }
    
    ComPorts[Port].dcb.BaudRate = Baud;
    
    ComPorts[Port].dcb.ByteSize=8;
    ComPorts[Port].dcb.StopBits=ONESTOPBIT;
    ComPorts[Port].dcb.Parity=NOPARITY;
    
    if(!SetCommState(ComPorts[Port].hPort, &ComPorts[Port].dcb))
    {
      //error setting serial port state
    }	
	}
	

/*=----------------------------------------------------------------------=*\
   Use :Reads a character from the COM port
       :Returns read byte, timeout_val if no data (timeout)
\*=----------------------------------------------------------------------=*/
	DECLDIR unsigned int ReadCharCOM( unsigned char Port, unsigned int timeout, unsigned int timeout_val )
	{
	  unsigned char data[2] = {255};
	  DWORD BytesRead= 0;
	
    COMMTIMEOUTS timeouts={0};
    timeouts.ReadIntervalTimeout=timeout;
    timeouts.ReadTotalTimeoutConstant=timeout;
    timeouts.ReadTotalTimeoutMultiplier=10; 
    timeouts.WriteTotalTimeoutConstant=50;
    timeouts.WriteTotalTimeoutMultiplier=10;
    SetCommTimeouts(ComPorts[Port].hPort, &timeouts);
	
	  ReadFile(ComPorts[Port].hPort, data, 1, &BytesRead, NULL);
	  
	  if (BytesRead)
	    return(data[0]);
	  else
	    return(timeout_val);
	}
	

/*=----------------------------------------------------------------------=*\
   Use :Transmits a character from the COM port
       :
\*=----------------------------------------------------------------------=*/
	DECLDIR void SendCharCOM( unsigned char Port, unsigned char Data )
	{
	  char data[2] = {0};
	  DWORD BytesRead= 0;
	  
	  data[0] = Data;
	  
	  WriteFile(ComPorts[Port].hPort, data, 1, &BytesRead, NULL);
	}
	

/*=----------------------------------------------------------------------=*\
   Use :Sets the DTR status for the currently open COM
       :
\*=----------------------------------------------------------------------=*/
	DECLDIR void SetDTRStatePrivate( unsigned char Port, unsigned char state )
	{
	  if (state)
	    ComPorts[Port].dcb.fDtrControl = DTR_CONTROL_ENABLE;
	  else
	    ComPorts[Port].dcb.fDtrControl = DTR_CONTROL_DISABLE;
      
    if(!SetCommState(ComPorts[Port].hPort, &ComPorts[Port].dcb))
    {
      //error setting serial port state
    }	
	}
	

/*=----------------------------------------------------------------------=*\
   Use :Sets the flow-control bit for the currently open COM
       :
\*=----------------------------------------------------------------------=*/
	DECLDIR void SetFlowControlPrivate( unsigned char Port, unsigned char state )
	{
	  if (state)
	  {
	    ComPorts[Port].dcb.fOutxCtsFlow = TRUE;
	    ComPorts[Port].dcb.fRtsControl = RTS_CONTROL_HANDSHAKE;
	  }
	  else
	  {
	    ComPorts[Port].dcb.fOutxCtsFlow = FALSE;
	    ComPorts[Port].dcb.fRtsControl = RTS_CONTROL_DISABLE;
	  }
      
    if(!SetCommState(ComPorts[Port].hPort, &ComPorts[Port].dcb))
    {
      //error setting serial port state
    }	
	}	
	
}